<?php
/*
Plugin Name: Trailmeme for WordPress
Plugin URI: http://trailmeme.com/downloads
Description: Trailmeme for WordPress (formerly WordTrails) allows for the creation of multi-branched trails of navigation through WordPress Blogs
Author: Jesse Silverstein (Xerox Corporation)
Version: 0.9.7.1
Author URI: http://blog.trailmeme.com/
*/

/*  Copyright 2008-2009  Xerox Corporation  (email : jesse.silverstein@xerox.com)

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

*/

/**
 * Note: For compactness, the TrailViewer source files (.fla and .as) are not
 * included in this package. They can be downloaded from
 * <http://open.xerox.com/xeroxproject/xerox-trails>
 */

/**
 * This is the initial file for the Trailmeme for WordPress plugin.
 *
 * This file contains all of the initialization functions for Trailmeme for WordPress. It
 * adds all of the action and filter hooks necesary.
 *
 * @package Trailmeme for WordPress
 * @since 0.0.1
 * @author Jesse Silverstein <jesse.silverstein@xerox.com>
 * @copyright 2008-2009 XIG: SemPrint
 * @global WordPress $wpdb WordPress database management class
 */

require_once "includes/WordTrailsGlobal.inc";

// {{{ wp_wt_install()

/**
 * Registers the Trailmeme for WordPress plugin with WordPress
 *
 * @global WordPress $wpdb WordPress database management class
 */
function wp_wt_install() {
    global $wpdb, $wp_rewrite;
    WordTrailsUtilities::updateNewOptions();
    WordTrailsGlobal::destroyFirst();

    require_once "includes/dbstruct.inc";
    
    WordTrailsUtilities::scheduleAnalytics();
    WordTrailsUtilities::pushDefaultControls();
    WordTrailsUtilities::addCapabilities();
    $wp_rewrite->flush_rules();
}

function wp_wt_deactivate() {
    WordTrailsUtilities::unScheduleAnalytics();
    WordTrailsUtilities::delCapabilities();
    
    global $wp_rewrite;
    $wp_rewrite->flush_rules();
}

// }}}
register_activation_hook(__FILE__,'wp_wt_install');
register_deactivation_hook(__FILE__, 'wp_wt_deactivate');

function wp_wt_add_weekly_schedule($schedules = array()) {
    $weekly = array('weekly' => array( 'interval' => 604800, 'display' => __('Once Weekly')));
    return array_merge($schedules, $weekly);
}

function wp_wt_edit_form_advanced() {
    global $post;
    add_meta_box("trail_blazing", "WordTrails", array("WordTrailsGlobal", "editFormBox"), "post", "advanced");
}

function wp_wt_widget_init() {
    register_sidebar_widget('Trailmeme Navigation', 'wp_wt_widget_worms_eye_view');
}
function wp_wt_widget_worms_eye_view($args) {
    global $wp_query;
    if ((is_single() || is_page()) && !is_page(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) {
        WordTrailsUtilities::makeWidget($wp_query->post->ID,$args);
    }
}

function wp_wt_template_redirect($args) {
    if (is_null(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) return;
    if (is_page(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) {
	include "includes/hijack.inc";
    }
}

function wp_wt_head() {
    WordTrailsUtilities::setStatics();
    wp_register_script("ltoh", WordTrailsGlobal::$urlpath . "js/ltoh.js", array("jquery"));
    $sidebars = get_option('sidebars_widgets');
    if (is_page(WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION))) {
	if (!empty($_GET["print"])) {
	    wp_enqueue_script("jquery-ui-draggable");
	    wp_enqueue_script("jquery-ui-sortable");
	}
	include_once "includes/tooltip.inc";
	wp_enqueue_script("ltoh");
	//wp_enqueue_script("WordTrailsWidget", WordTrailsGlobal::$urlpath . "js/widget.js", array("jquery", "tooltip"));
	if (function_exists("wp_enqueue_style")) {
	    wp_enqueue_style("WordTrailsIndex", WordTrailsGlobal::$urlpath . "css/trailindex.css");
	} else {
	    add_action("wp_head", "wp_wt_index_css");
	}
    } elseif (is_array($sidebars)) {
	foreach ($sidebars as $bar) {
	    if (in_array('trailmeme-navigation', (array)$bar)) {
		include_once "includes/tooltip.inc";
		wp_enqueue_script("WordTrailsWidget", WordTrailsGlobal::$urlpath . "js/widget.js", array("ltoh"));
		if (function_exists("wp_enqueue_style")) {
		    wp_enqueue_style("WordTrailsWidget", WordTrailsGlobal::$urlpath . "css/widget.css");
		} else {
		    add_action("wp_head", "wp_wt_widget_css");
		}
		break;
	    }
	}
    }
}

function wp_wt_widget_css() {
    WordTrailsUtilities::setStatics();?>

<link rel="stylesheet" type="text/css" href="<?php echo WordTrailsGlobal::$urlpath; ?>css/widget.css" /><?php
}

function wp_wt_index_css() {
    WordTrailsUtilities::setStatics();?>

<link rel="stylesheet" type="text/css" href="<?php echo WordTrailsGlobal::$urlpath; ?>css/trailindex.css" /><?php
}

function wp_wt_track_hits() {
    global $wp_the_query;
    if (is_single()||is_page()) WordTrailsGlobal::trackHits($wp_the_query->post->ID);
}

function wp_wt_admin_init() {
    WordTrailsUtilities::setStatics();
}

function wp_wt_slug_tags() {
    /*
    global $wp_rewrite;
    $index_pid = WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION);
    $index_slug = get_post_field('post_name', $index_pid);
    add_rewrite_rule($index_slug . "/(.+?)/?$", "index.php?page_id=" . $index_pid . "&trail_slug=\$matches[1]", "top");
    
    $permastruct = get_option('permalink_structure');
    $structs = split("/", $permastruct);
    $pattern = "";
    $redirect = "index.php?";
    $i=1;
    foreach ($structs as $struct) {
	if (empty($struct)) continue;
	$structid = array_search($struct, $wp_rewrite->rewritecode);
	if (false === $structid) {
	    $pattern .= $struct . "/";
	    continue;
	}
	$pattern .= $wp_rewrite->rewritereplace[$structid] . "/";
	$redirect .= $wp_rewrite->queryreplace[$structid] . "\$matches[$i]&";
	$i++;
    }
    add_rewrite_rule($pattern . "on-trail/(.+?)/?$", $redirect . "trail_slug=\$matches[$i]","top");
    add_rewrite_rule("(.+?)/on-trail/(.+?)/?$", "index.php?pagename=\$matches[1]&trail_slug=\$matches[2]", "top");
    
    foreach (array("xml", "print") as $type) {
	add_rewrite_rule(WordTrailsGlobal::$redirect_query_slugs[$type] . "/(.+?)/?$","index.php?wt_action=" . WordTrailsGlobal::$redirect_query_strings[$type] . "&trail_slug=\$matches[1]","top");
    }
    */
    add_rewrite_tag("%trail_slug%", ".*");
    add_rewrite_tag("%wt_action%", "wt_.*");
}

function wp_wt_slug_rules_filter($rules) {
    global $wp_rewrite;
    $newrules = array();
    
    $index_pid = WordTrailsGlobal::get_option(WordTrailsGlobal::TRAILS_PAGE_OPTION);
    $index_slug = get_post_field('post_name', $index_pid);
    $newrules[$index_slug . "/(.+?)/?$"] = "index.php?page_id=" . $index_pid . "&trail_slug=\$matches[1]";
    
    $permastruct = get_option('permalink_structure');
    $structs = split("/", $permastruct);
    $pattern = "";
    $redirect = "index.php?";
    $i=1;
    foreach ($structs as $struct) {
	if (empty($struct)) continue;
	$structid = array_search($struct, $wp_rewrite->rewritecode);
	if (false === $structid) {
	    $pattern .= $struct . "/";
	    continue;
	}
	$pattern .= $wp_rewrite->rewritereplace[$structid] . "/";
	$redirect .= $wp_rewrite->queryreplace[$structid] . "\$matches[$i]&";
	$i++;
    }
    $newrules[$pattern . "on-trail/(.+?)/?$"] = $redirect . "trail_slug=\$matches[$i]";
    $newrules["(.+?)/on-trail/(.+?)/?$"] = "index.php?pagename=\$matches[1]&trail_slug=\$matches[2]";
    
    foreach (array("xml", "print") as $type) {
	$newrules[WordTrailsGlobal::$redirect_query_slugs[$type] . "/(.+?)/?$"] = "index.php?wt_action=" . WordTrailsGlobal::$redirect_query_strings[$type] . "&trail_slug=\$matches[1]";
    }
    
    return $newrules + $rules;
}

add_action('admin_menu', array("WordTrailsUtilities", "installMenus"));
add_action('shutdown', array("WordTrailsGlobal", "sessionStore"));
add_action('get_header', 'wp_wt_head');
add_action('loop_start', 'wp_wt_track_hits');
add_action('template_redirect', 'wp_wt_template_redirect');
add_action('parse_request', array("WordTrailsGlobal", "capture_redirect"));
add_action('edit_form_advanced', 'wp_wt_edit_form_advanced');
add_action('save_post', array("WordTrailsGlobal", "blazeTrail"));
add_action('delete_post', array("WordTrailsGlobal", "deletePID"));
add_action('widgets_init', 'wp_wt_widget_init');
add_filter('cron_schedules', 'wp_wt_add_weekly_schedule');
add_action(WordTrailsGlobal::ANALYTICS_TRANSMIT_OPTION, array("WordTrailsGlobal", "sendAnalyticsToTrailMeme"));
add_action('set_current_user', array("WordTrailsUtilities", "setStatics"));
add_action("admin_init", "wp_wt_admin_init");
add_filter("init", "wp_wt_slug_tags");
add_filter("rewrite_rules_array", "wp_wt_slug_rules_filter");

if (defined("DOING_AJAX"))
    if (DOING_AJAX)
        require_once "includes/ajax.inc";
?>